@@ -76,8 +76,16 @@ config.diagnostics.groupFileStatus =
 设置为 `Fallback` 意味着组中的诊断由 `diagnostics.neededFileStatus` 单独设置。
 其他设置将覆盖单独设置，但是不会覆盖以 `!` 结尾的设置。
 ]]
+config.diagnostics.workspaceEvent =
+"设置触发工作区诊断的时机。"
+config.diagnostics.workspaceEvent.OnChange =
+"当文件发生变化时触发工作区诊断。"
+config.diagnostics.workspaceEvent.OnSave =
+"当文件保存时触发工作区诊断。"
+config.diagnostics.workspaceEvent.None =
+"关闭工作区诊断。"
 config.diagnostics.workspaceDelay =
-"进行工作区诊断的延迟（毫秒）。当你启动工作区，或编辑了任意文件后，将会在后台对整个工作区进行重新诊断。设置为负数可以禁用工作区诊断。"
+"进行工作区诊断的延迟（毫秒）。"
 config.diagnostics.workspaceRate  =
 "工作区诊断的运行速率（百分比）。降低该值会减少CPU占用，但是也会降低工作区诊断的速度。你当前正在编辑的文件的诊断总是全速完成，不受该选项影响。"
 config.diagnostics.libraryFiles   =
@@ -262,6 +270,8 @@ config.telemetry.enable                  =
 ]]
 config.misc.parameters                   =
 'VSCode中启动语言服务时的[命令行参数](https://github.com/sumneko/lua-language-server/wiki/Getting-Started#arguments)。'
+config.misc.executablePath               =
+'VSCode中指定可执行文件路径。'
 config.IntelliSense.traceLocalSet        =
 '请查阅[文档](https://github.com/sumneko/lua-language-server/wiki/IntelliSense-optional-features)了解用法。'
 config.IntelliSense.traceReturn          =
@@ -284,6 +294,12 @@ config.type.weakNilCheck                 =
 
 此设置为 `false` 时，`numer|nil` 类型无法赋给 `number` 类型；为 `true` 是则可以。
 ]]
+config.doc.privateName                   =
+'将特定名称的字段视为私有，例如 `m_*` 意味着 `XXX.m_id` 与 `XXX.m_type` 是私有字段，只能在定义所在的类中访问。'
+config.doc.protectedName                 =
+'将特定名称的字段视为受保护，例如 `m_*` 意味着 `XXX.m_id` 与 `XXX.m_type` 是受保护的字段，只能在定义所在的类极其子类中访问。'
+config.doc.packageName                   =
+'将特定名称的字段视为package，例如 `m_*` 意味着 `XXX.m_id` 与 `XXX.m_type` 只能在定义所在的文件中访问。'
 config.diagnostics['unused-local']          =
 '未使用的局部变量'
 config.diagnostics['unused-function']       =
